@@ -13,10 +13,10 @@ class API:
         self.metadata = metadata
 
     @staticmethod
-    def identify(id=None, name=None):
+    def identify(thing_id=None, thing_name=None):
         """
         Returns an identifier with filled with id or name. If both are filled, will prefer id.
         """
-        if not id and not name:
-            raise Exception("Either `name` or `id` needs to be specified")
-        return stypes.Identifier(id=id) if id else stypes.Identifier(name=name)
+        if not thing_id and not thing_name:
+            raise Exception("Either `thing_name` or `thing_id` needs to be specified")
+        return stypes.Identifier(id=thing_id) if thing_id else stypes.Identifier(name=thing_name)
