@@ -4,7 +4,7 @@ import os
 from typing import *
 
 from cognite.seismic._api.api import API
-from cognite.seismic.data_classes.api_types import Survey
+from cognite.seismic.data_classes.api_types import Survey, SurveyGridTransformation
 from cognite.seismic.data_classes.errors import SeismicServiceError
 from grpc import StatusCode
 
@@ -29,6 +29,7 @@ class SurveyV1API(API):
         include_metadata: bool = False,
         crs: Optional[str] = None,
         in_wkt: Optional[bool] = None,
+        include_grid_transformation: Optional[bool] = False,
     ):
         """List all the surveys.
         Provide either crs or in_wkt to get surveys' coverage.
@@ -39,12 +40,15 @@ class SurveyV1API(API):
             include_metadata (bool): true if metadata should be included in the response.
             crs(str): the crs in which the surveys' coverage is returned, default is original survey crs
             in_wkt(bool): surveys' coverage format, set to true if wkt format is needed, default is geojson format
+            include_grid_transformation (bool): if set to True, return the user-specified transformation between bin grid and projected coordinates
 
         Returns:
             List[Survey]: the requested surveys and their files (if requested).
         """
 
-        return self._search_internal([], list_seismics, list_seismic_stores, include_metadata, crs, in_wkt)
+        return self._search_internal(
+            [], list_seismics, list_seismic_stores, include_metadata, crs, in_wkt, include_grid_transformation
+        )
 
     def search(
         self,
@@ -55,6 +59,7 @@ class SurveyV1API(API):
         include_metadata: bool = False,
         crs: Optional[str] = None,
         in_wkt: Optional[bool] = None,
+        include_grid_transformation: Optional[bool] = False,
     ):
         """Search for subset of surveys.
         Provide either crs or in_wkt to get surveys' coverage.
@@ -67,6 +72,7 @@ class SurveyV1API(API):
             include_metadata (bool): true if metadata should be included in the response.
             crs(str): the crs in which the surveys' coverage is returned, default is original survey crs
             in_wkt(bool): surveys' coverage format, set to true if wkt format is needed, default is geojson format
+            include_grid_transformation (bool): if set to True, return the user-specified transformation between bin grid and projected coordinates
 
         Returns:
             List[Survey]: the requested surveys and their files (if requested).
@@ -80,7 +86,9 @@ class SurveyV1API(API):
         if survey_external_id_substring is not None:
             search_specs.append(SearchSpec(external_id_substring=survey_external_id_substring))
 
-        return self._search_internal(search_specs, list_seismics, list_seismic_stores, include_metadata, crs, in_wkt)
+        return self._search_internal(
+            search_specs, list_seismics, list_seismic_stores, include_metadata, crs, in_wkt, include_grid_transformation
+        )
 
     def get(
         self,
@@ -92,6 +100,7 @@ class SurveyV1API(API):
         include_metadata: bool = False,
         crs: Optional[str] = None,
         in_wkt: Optional[bool] = None,
+        include_grid_transformation: Optional[bool] = False,
     ):
         """
         Get a survey by either id, external_id or name.
@@ -106,6 +115,7 @@ class SurveyV1API(API):
             include_metadata (bool): true if metadata should be included in the response.
             crs(str): the crs in which the survey coverage is returned, default is original survey crs
             in_wkt(bool): survey coverage format, set to true if wkt format is needed, default is geojson format
+            include_grid_transformation (bool): if set to True, return the user-specified transformation between bin grid and projected coordinates
 
         Returns:
             Survey: the requested survey, its seismics, seismic stores and metadata (if requested).
@@ -121,37 +131,81 @@ class SurveyV1API(API):
         else:
             search_spec = SearchSpec(name=survey_name)
 
-        result = self._search_internal([search_spec], list_seismics, list_seismic_stores, include_metadata, crs, in_wkt)
+        result = self._search_internal(
+            [search_spec],
+            list_seismics,
+            list_seismic_stores,
+            include_metadata,
+            crs,
+            in_wkt,
+            include_grid_transformation,
+        )
         if len(result) == 0:
             raise SeismicServiceError(StatusCode.NOT_FOUND, "survey not found")
         else:
             return result[0]
 
-    def register(self, survey_name: str, metadata: dict = None):
+    def register(
+        self,
+        survey_name: str,
+        metadata: dict = None,
+        external_id: Optional[str] = None,
+        crs: Optional[str] = None,
+        grid_transformation: Optional[SurveyGridTransformation] = None,
+    ):
         """Finds surveys for which the coverage area intersects with the given set of coordinates or exact metadata key-value match.
 
         Args:
             survey_name (str): survey name.
             metadata (dict): metadata of the survey.
+            external_id: external id of the survey.
+            crs (str): Coordinate reference system to be used by all
+                                 members of this survey
+            grid_transformation (SurveyGridTransformation):
+                Manually specify an affine transformation between bin grid
+                coordinates and projected crs coordinates, either using an
+                origin point and the azimuth of the xline axis
+                (:py:class:`~cognite.seismic.data_classes.api_types.P6Transformation`)
+                or by specifying three or more corners of the grid as a list of
+                :py:class:`~cognite.seismic.data_classes.api_types.DoubleTraceCoordinates`.
+                This transformation must be valid for all members of this survey.
 
         Returns:
             RegisterSurveyResponse: id, name and metadata of the survey.
         """
-        return self.v0_survey_api.register(survey_name, metadata)
+        return self.v0_survey_api.register(survey_name, metadata, external_id, crs, grid_transformation)
 
-    def edit(self, survey_id: Optional[str] = None, survey_name: Optional[str] = None, metadata: dict = None):
+    def edit(
+        self,
+        survey_id: Optional[str] = None,
+        survey_name: Optional[str] = None,
+        metadata: dict = None,
+        external_id: Optional[str] = None,
+        crs: Optional[str] = None,
+        grid_transformation: Optional[SurveyGridTransformation] = None,
+    ):
         """Edit a survey
 
         Args:
             survey_id (Optional[str]): id of the survey to edit.
             survey_name (Optional[str]): name of the survey to edit.
             metadata (dict): metadata of the survey to edit.
+            crs (Optional[str]): Coordinate reference system to be used by all
+                                 members of this survey
+            grid_transformation (Optional[SurveyGridTransformation]):
+                Manually specify an affine transformation between bin grid
+                coordinates and projected crs coordinates, either using an
+                origin point and the azimuth of the xline axis
+                (:py:class:`~cognite.seismic.data_classes.api_types.P6Transformation`)
+                or by specifying three or more corners of the grid as a list of
+                :py:class:`~cognite.seismic.data_classes.api_types.DoubleTraceCoordinates`.
+                This transformation must be valid for all members of this survey.
 
         Returns:
             EditSurveyResponse: id, name and metadata of the survey.
 
         """
-        return self.v0_survey_api.edit(survey_id, survey_name, metadata)
+        return self.v0_survey_api.edit(survey_id, survey_name, metadata, external_id, crs, grid_transformation)
 
     def delete(self, survey_id: Optional[str] = None, survey_name: Optional[str] = None):
         """Delete a survey
@@ -174,6 +228,7 @@ class SurveyV1API(API):
         include_metadata: bool = False,
         crs: Optional[str] = None,
         in_wkt: Optional[bool] = None,
+        include_grid_transformation: Optional[bool] = False,
     ):
         coverageParamCrs = CRS(crs=crs) if crs is not None else None
         coverageParams = (
@@ -187,6 +242,7 @@ class SurveyV1API(API):
             list_seismic_store_ids=list_seismic_stores,
             include_metadata=include_metadata,
             include_coverage=coverageParams,
+            include_grid_transformation=include_grid_transformation,
         )
         return [
             Survey.from_proto(survey_proto)
