@@ -54,7 +54,10 @@ def get_search_spec(
     """
     spec = SearchSpec()
     if id is not None:
-        spec.id = id
+        if isinstance(id, int):
+            spec.id = id
+        else:
+            spec.id_string = id
     if external_id is not None:
         spec.external_id = external_id
     if external_id_substring is not None:
