@@ -3,7 +3,9 @@ import logging
 import os
 import sys
 import time
+from collections import namedtuple
 
+import deprecation
 import grpc
 from grpc._channel import (
     _InactiveRpcError,
@@ -104,6 +106,31 @@ def decorate_with_wrapped_rpc_errors(*args):
             setattr(obj, key, with_wrapped_rpc_errors(attr))
 
 
+ParsedUrl = namedtuple("ParsedUrl", "is_insecure target")
+
+
+def parse_url_parameter(url) -> ParsedUrl:
+    """Parse a provided url, setting the secure/insecure status and standardizing the URL with a port
+    """
+    from urllib.parse import urlparse
+
+    o = urlparse(url)
+    # Urllib parsing quirk: if there's no schema, it parses the entire url as a path
+    if o.netloc:
+        netloc = o.netloc
+    else:
+        netloc = o.path
+
+    scheme = o.scheme or "https"
+    # We're removing any potential port that might exist, esp since it's also provided separately.
+    # The rest of the URL (the path and params) are discarded
+    base_url = netloc.split(":")[0] or "api.cognitedata.com"
+    port = o.port or 443
+    full_url = f"{base_url}:{port}"
+
+    return ParsedUrl(scheme == "http", full_url)
+
+
 class CogniteSeismicClient:
     """
     Main class for the seismic client
@@ -118,9 +145,18 @@ class CogniteSeismicClient:
             raise ValueError(
                 "You have either not passed an api key or not set the COGNITE_API_KEY environment variable."
             )
-        self.base_url = base_url or "api.cognitedata.com"
-        self.port = port or "443"
-        self.url = self.base_url + ":" + str(self.port)
+
+        if port is not None or insecure:
+            logger.warning("The port and insecure params are deprecated. Please include them in the base_url.")
+
+        if base_url is not None:
+            parsed_url = parse_url_parameter(base_url)
+            self.target = parsed_url.target
+            self.is_insecure = parsed_url.is_insecure
+        else:
+            self.target = "api.cognitedata.com:443"
+            self.is_insecure = False
+
         self.metadata = [("api-key", self.api_key)]
 
         if self.project is not None:
@@ -138,11 +174,11 @@ class CogniteSeismicClient:
             ("grpc.http2.max_pings_without_data", 0),
             ("grpc.http2.min_time_between_pings_ms", 5000),
         ]
-        if insecure:
-            channel = grpc.insecure_channel(self.url, options=options)
+        if self.is_insecure:
+            channel = grpc.insecure_channel(self.target, options=options)
         else:
             credentials = grpc.ssl_channel_credentials(root_certificates=custom_root_cert)
-            channel = grpc.secure_channel(self.url, credentials, options=options)
+            channel = grpc.secure_channel(self.target, credentials, options=options)
         self.query = query_serv.QueryStub(channel)
         self.ingestion = ingest_serv.IngestStub(channel)
         self.seismicstub = seismic_serv.SeismicAPIStub(channel)
