@@ -1,7 +1,8 @@
 import json
 import math
 import os
-from typing import Iterable, List, Mapping, Optional, Tuple, Union
+from enum import Enum
+from typing import Iterable, List, Mapping, NamedTuple, Optional, Tuple, Union
 
 import numpy as np
 from cognite.seismic._api.utility import Direction, LineRange, MaybeString
@@ -11,11 +12,18 @@ if not os.getenv("READ_THE_DOCS"):
     from cognite.seismic.protos.types_pb2 import GeoJson
     from cognite.seismic.protos.types_pb2 import Geometry as GeometryProto
     from cognite.seismic.protos.types_pb2 import IngestionSource, Wkt
+    from cognite.seismic.protos.types_pb2 import (
+        P6Transformation as P6TransformationProto,
+        DoubleTraceCoordinates as DoubleTraceCoordinatesProto,
+        DeduceFromTraces as DeduceFromTracesProto,
+        SurveyGridTransformation as SurveyGridTransformationProto,
+        TraceCorners,
+    )
     from cognite.seismic.protos.v1.seismic_service_datatypes_pb2 import BinaryHeader as BinaryHeaderProto
     from cognite.seismic.protos.v1.seismic_service_datatypes_pb2 import TextHeader as TextHeaderProto
     from google.protobuf.json_format import MessageToDict
+
 else:
-    from enum import Enum
 
     class IngestionSource(Enum):
         """Enum of ingestion sources."""
@@ -469,23 +477,33 @@ class File:
     name: str
     metadata: Mapping[str, str]
     is_temporary: bool
+    external_id: Optional[str]
 
-    def __init__(self, *, id, name, metadata, is_temporary):
+    def __init__(self, *, id, name, metadata, is_temporary, external_id):
         self.id = id
         self.name = name
         self.metadata = metadata
         self.is_temporary = is_temporary
+        self.external_id = external_id
 
     def __repr__(self):
         temporary = ", temporary" if self.is_temporary else ""
-        return f"File<id: {self.id}, name: {self.name}{temporary}, metadata: {metadata}>"
+        return (
+            f"File<id: {self.id}, name: {self.name}{temporary}, metadata: {metadata}, external_id: {self.external_id}>"
+        )
 
     @staticmethod
     def from_proto(proto) -> "File":
         metadata = {}
         for key in proto.metadata:
             metadata[key] = proto.metadata[key]
-        return File(id=proto.id, name=proto.name, metadata=metadata, is_temporary=proto.is_temporary)
+        return File(
+            id=proto.id,
+            name=proto.name,
+            metadata=metadata,
+            is_temporary=proto.is_temporary,
+            external_id=proto.external_id,
+        )
 
 
 class Geometry:
@@ -986,6 +1004,132 @@ class Partition:
         return Partition(id=proto.id, external_id=proto.external_id, name=proto.name, seismics=seismics)
 
 
+class DoubleTraceCoordinates(NamedTuple):
+    """The coordinates of a point, given in both projected coordinates (x, y) and in
+    bin grid coordinates (inline, xline).
+
+    Pass a list of three or more instances to specify the affine transformation between bin
+    grid coordinates and projected coordinates by the corners of the grid.
+
+    Attributes:
+        x (float): Longitude or easting
+        y (float): Latitude or northing
+        inline (int): The inline coordinate of the bin grid
+        xline (int): The xline coordinate of the bin grid
+    """
+
+    x: float
+    y: float
+    inline: int
+    xline: int
+
+    def to_proto(self):
+        return DoubleTraceCoordinatesProto(x=self.x, y=self.y, iline=self.inline, xline=self.xline)
+
+    @staticmethod
+    def from_proto(proto) -> "DoubleTraceCoordinates":
+        return DoubleTraceCoordinates(x=proto.x, y=proto.y, inline=proto.iline, xline=proto.xline)
+
+
+class Handedness(Enum):
+    """The handedness of a :py:class:`~cognite.seismic.data_classes.api_types.P6Transformation`.
+    RIGHTHANDED means that the inline axis is 90 degrees clockwise from the xline axis. 
+    LEFTHANDED means that the inline axis is 90 degrees counter-clockwise from the xline axis."""
+
+    RIGHTHANDED = 0
+    LEFTHANDED = 1
+
+
+class P6Transformation(NamedTuple):
+    """Description of an affine transformation between bin grid coordinates and projected 
+    coordinates in the P6 format, loosely following IOGP guidance note 373-7-2 section 2.3.2.4.
+
+    Attributes:
+        origin (:py:class:`~cognite.seismic.data_classes.api_types.DoubleTraceCoordinates`):
+            The origin point of the transformation
+        inline_bin_width (float): The width of bins in the inline direction
+        xline_bin_width (float): The width of bins in the xline direction
+        xline_azimuth (float): The direction of the xline axis, in degrees clockwise from north
+        handedness (Optional[Handedness]):
+            The orientation of the inline vs. the xline axes. Default: RIGHTHANDED.
+        inline_bin_inc (Optional[int]):
+            The increment of the inline coordinate corresponding to a bin. Default: 1.
+        xline_bin_inc (Optional[int]):
+            The increment of the xline coordinate corresponding to a bin. Default: 1.
+    """
+
+    origin: DoubleTraceCoordinates
+    inline_bin_width: float
+    xline_bin_width: float
+    xline_azimuth: float
+    handedness: Optional[Handedness] = Handedness.RIGHTHANDED
+    inline_bin_inc: Optional[int] = 1
+    xline_bin_inc: Optional[int] = 1
+
+    def to_proto(self):
+        return P6TransformationProto(
+            handedness=self.handedness.value,
+            origin=self.origin.to_proto(),
+            iline_bin_width=self.inline_bin_width,
+            xline_bin_width=self.xline_bin_width,
+            xline_azimuth=self.xline_azimuth,
+            iline_bin_inc=self.inline_bin_inc,
+            xline_bin_inc=self.xline_bin_inc,
+        )
+
+    @staticmethod
+    def from_proto(proto) -> "P6Transformation":
+        return P6Transformation(
+            origin=DoubleTraceCoordinates.from_proto(proto.origin),
+            handedness=Handedness(proto.handedness),
+            inline_bin_width=proto.iline_bin_width,
+            xline_bin_width=proto.xline_bin_width,
+            xline_azimuth=proto.xline_azimuth,
+            inline_bin_inc=proto.iline_bin_inc,
+            xline_bin_inc=proto.xline_bin_inc,
+        )
+
+
+class DeduceFromTraces(NamedTuple):
+    """Pass an instance of this class as the grid_transformation argument to indicate that
+    the seismics service should try to deduce the affine transformation between bin grid
+    coordinates and projected coordinates from the trace coordinates themselves."""
+
+    def to_proto(self):
+        return DeduceFromTracesProto()
+
+
+SurveyGridTransformation = Union[P6Transformation, List[DoubleTraceCoordinates], DeduceFromTraces]
+
+
+def survey_grid_transformation_to_proto(grid_transformation: Optional[SurveyGridTransformation] = None):
+    if grid_transformation is None:
+        return None
+    elif isinstance(grid_transformation, P6Transformation):
+        return SurveyGridTransformationProto(p6_transformation=grid_transformation.to_proto())
+    elif isinstance(grid_transformation, list):
+        return SurveyGridTransformationProto(
+            trace_corners=TraceCorners(corners=[c.to_proto() for c in grid_transformation])
+        )
+    elif isinstance(grid_transformation, DeduceFromTraces):
+        return SurveyGridTransformationProto(deduce_from_traces=grid_transformation.to_proto())
+    else:
+        raise ValueError("Invalid SurveyGridTransformation")
+
+
+def survey_grid_transformation_from_proto(proto) -> SurveyGridTransformation:
+    if proto is None:
+        return None
+    elif proto.HasField("p6_transformation"):
+        return P6Transformation.from_proto(proto.p6_transformation)
+    elif proto.HasField("trace_corners"):
+        return [DoubleTraceCoordinates.from_proto(p) for p in proto.trace_corners.corners]
+    elif proto.HasField("deduce_from_traces"):
+        return DeduceFromTraces()
+    else:
+        raise ValueError("Invalid SurveyGridTransformation protobuf")
+
+
 class Survey:
 
     id: str
@@ -995,8 +1139,12 @@ class Survey:
     seismic_store_ids: Optional[List[int]]
     metadata: Optional[Mapping[str, str]]
     coverage: Optional[Geometry]
+    crs: Optional[str]
+    grid_transformation: Optional[SurveyGridTransformation]
 
-    def __init__(self, id, external_id, name, seismic_ids, seismic_store_ids, metadata, coverage):
+    def __init__(
+        self, id, external_id, name, seismic_ids, seismic_store_ids, metadata, coverage, crs, grid_transformation
+    ):
         self.id = id
         self.external_id = external_id
         self.name = name
@@ -1004,18 +1152,27 @@ class Survey:
         self.seismic_store_ids = seismic_store_ids
         self.metadata = metadata
         self.coverage = coverage
+        self.crs = crs
+        self.grid_transformation = grid_transformation
 
     @staticmethod
     def from_proto(proto) -> "Survey":
         survey_id = proto.survey.id
-        external_id = None
+        external_id = proto.survey.external_id if hasattr(proto.survey, "external_id") else None
         name = proto.survey.name if hasattr(proto.survey, "name") else None
         metadata = proto.survey.metadata if hasattr(proto.survey, "metadata") else None
         seismic_ids = proto.seismic_ids if hasattr(proto, "seismic_ids") else None
         seismic_store_ids = proto.seismic_store_ids if hasattr(proto, "seismic_store_ids") else None
         coverage = Geometry.from_proto(proto.coverage) if proto.HasField("coverage") else None
-
-        return Survey(survey_id, external_id, name, seismic_ids, seismic_store_ids, metadata, coverage)
+        crs = proto.survey.crs if proto.survey.crs else None
+        grid_transformation = (
+            survey_grid_transformation_from_proto(proto.survey.grid_transformation)
+            if proto.survey.HasField("grid_transformation")
+            else None
+        )
+        return Survey(
+            survey_id, external_id, name, seismic_ids, seismic_store_ids, metadata, coverage, crs, grid_transformation
+        )
 
     def __repr__(self):
         return "%s {\n%s\n}" % (type(self).__name__, "\n".join("%s = %s" % item for item in vars(self).items()))
